// Instructables Make It Glow 2012 HAT Sequencer
// ARDUINO UNO Rev. 3 + SPARKFUN EL Escudo Dos
//
//   ================= PARTS and CONFIG ===================
// EL Escudo Dos:        Shield mounted on Arduino UNO Rev. 3
// EL Wire:              Connected to El Escudo Dos on Digital out Digital Pins 2 -> 9(OUTPUT)
// LED indicator:        Indicates mode change and sound detection Pin 13(OUTPUT)
// Momentary switch:     Mode switcher (usually open) connected to Digital Pin 11(INPUT)
// Potentiometer:        Speed/Sound Control connected to Analog 0
// DFRobot Sound Sensor: Microphone Sound Sensor connected to Analog 2

// Writen by:    Dovadil

// DISCAIMER: I am a very amateur programmer, so please dont judge the probably unnecessarily complicated code.
//            I just had an idea and made it happen. I hope you can use bits and pieces of this for your own project.
//            SHARE IT !!!

//constants for the button press input and 1 led output
const int buttonPin = 11;     // the number of the pushbutton pin
const int ledPin =  13;      // the number of the LED pin

//const_variable for the pot and Sound Sensor
const int analogInPin = A0;  // Analog input pin that the potentiometer is attached to
const int analogInPinSound = A2;  // Analog input Sound Sensor
int potValue = 0;        // value read from the pot
int time = 100;          // time interval

//varibales
boolean initialise=true;
int buttonState = 0;         // variable for reading the pushbutton status
int mode = 0;
int soundVal = 0;
int soundPotThresholdValue = 0;
int soundSensitivityLimit=0;
int x;
int y;
int z;

void setup()
{ 
  // initialize serial communications at 9600 bps:
  Serial.begin(9600);  
  // The EL channels are on pins 2 through 9
  // Initialize the pins as outputs
  pinMode(2, OUTPUT);  // channel A
  pinMode(3, OUTPUT);  // channel B 
  pinMode(4, OUTPUT);  // channel C
  pinMode(5, OUTPUT);  // channel D  
  pinMode(6, OUTPUT);  // channel E
  pinMode(7, OUTPUT);  // channel F
  pinMode(8, OUTPUT);  // channel G
  pinMode(9, OUTPUT);  // channel H
  // We also have two status LEDs, pin 10 on the Escudo,
  pinMode(10, OUTPUT);   
  // and ledPin(pin 13) on the Arduino itself
  pinMode(ledPin, OUTPUT);
  pinMode(buttonPin, INPUT); 
}

void loop()
{
  //Start program with a single run of LED illumination
  if (initialise==true)
  {
    delay(10);
    Serial.println("");
    Serial.println("");    
    Serial.println("Initialising...");     
    Serial.println("");
    int status;
    // Step through all eight EL channels (pins 2 through 9)
    for (x=2; x<=9; x++)
    {
      digitalWrite(ledPin, HIGH);    // turn LED on
      digitalWrite(x, HIGH);         // turn the EL channel on
      delay(100);                    // wait for 1/10 second
      digitalWrite(ledPin, LOW);     // turn LED off
      digitalWrite(x, LOW);          // turn the EL channel off
      delay(50);                     // wait for 1/10 second
      digitalWrite(ledPin, HIGH);    // turn LED on
      digitalWrite(x, HIGH);         // turn the EL channel on
      delay(100);                    // wait for 1/10 second
      digitalWrite(ledPin, LOW);     // turn LED off
      digitalWrite(x, LOW);          // turn the EL channel off
      delay(50);
      //stops running of initialisation after one iteration
      initialise=false;
    }
    Serial.println("");    
    Serial.println("Initialisation DONE!");     
    Serial.println("Waiting for Mode trigger");
  }
  detail();

  //=================MODE 1(Step through all eight EL channels)==================
  if (mode==1)
  {
    // Step through all eight EL channels (pins 2 through 9)
    for (x=2; x<=9; x++)
    {
      detail();
      digitalWrite(x, HIGH);   // turn the EL channel on
      delay(time);              // wait for 1/10 second
      digitalWrite(x, LOW);    // turn the EL channel off
    }
  }

  //=================MODE 2(fill up from bottom then empty from bottom)==================
  if (mode==2)
  {
    for (x=2; x<=9; x++)
    {
      detail();
      digitalWrite(x, HIGH);   // turn the EL channel on
      delay(time);        // wait for 1/10 second
    }
    for (x=2; x<=9; x++)
    {
      detail();
      digitalWrite(x, LOW);   // turn the EL channel on
      delay(time);        // wait for 1/10 second
    }
  }

  //=================MODE 3(up down up down)==================
  if (mode==3)
  {
    for (x=2; x<=9; x++)
    {
      detail();
      digitalWrite(x, HIGH);   // turn the EL channel on
      delay(time);        // wait for 1/10 second
      digitalWrite(x, LOW);    // turn the EL channel off
    }
    for (x=2; x<=9; x++)
    {
      detail();
      y=11-x;
      digitalWrite(y, HIGH);   // turn the EL channel on
      delay(time);        // wait for 1/10 second
      digitalWrite(y, LOW);    // turn the EL channel off
    }
  }

  //=================MODE 4(sweep in bottom to top , sweep out top to bottom)==================
  if (mode==4)
  {
    for(x = 2; x <= 9; x++)
    {
      detail();
      digitalWrite(x, HIGH);
      delay(time);
    }
    for(x = 9; x >= 2; x--)
    {
      detail();
      digitalWrite(x, LOW);
      delay(time);
    }
  }
 
  //=================MODE 5(stack and unstack)==================
  if (mode==5)
  {
    for (x = 2; x <= 9; x++)
    {
      for (y = 9; y >= x; y--)
      {
        digitalWrite(y, HIGH);
        delay(time); 
        digitalWrite(y, LOW);
        detail();   
      }
      digitalWrite(x, HIGH);
    }
    for (x = 9; x >= 2; x--)
    {
      for (y = x; y <= 9; y++)
      {
        digitalWrite(y, HIGH); 
        delay(time);              
        digitalWrite(y, LOW);      
        detail();
      }
    }
    digitalWrite(y, LOW);
    delay(time+20);
  }
 
  //=================MODE 6(alternate flashing)==================
  if (mode==6)
  {
    detail();
    digitalWrite(2, HIGH);
    digitalWrite(4, HIGH);
    digitalWrite(6, HIGH);
    digitalWrite(8, HIGH);
    delay(time+75);
    detail();
    digitalWrite(2, LOW);
    digitalWrite(4, LOW);
    digitalWrite(6, LOW);
    digitalWrite(8, LOW);
    digitalWrite(3, HIGH);
    digitalWrite(5, HIGH);
    digitalWrite(7, HIGH);
    digitalWrite(9, HIGH);
    delay(time+75);
    detail();
    digitalWrite(3, LOW);
    digitalWrite(5, LOW);
    digitalWrite(7, LOW);
    digitalWrite(9, LOW);
  }

  //=================MODE 7(criss cross)==================
  if (mode==7)
  {
    for (x = 2; x <= 9; x++)
    {
      y=11-x;
      digitalWrite(x, HIGH);   // turn the LED on (HIGH is the voltage level)
      digitalWrite(y, HIGH);   // turn the LED on (HIGH is the voltage level)
      if (x==5)
      {
        delay(time/2);
      }
      else
      {
        delay(time);
      }
      digitalWrite(x, LOW);
      digitalWrite(y, LOW);
      detail();   
    }   
  }   

  //=================MODE 8(Sound Mode) ==================
  if (mode==8)
  {
    // Sound test
    detailSound();
    if ((soundVal,DEC) == 512)
    {
      //sound flash up
      for(x = 2; x <= 9; x++)
      {
        digitalWrite(x, HIGH);
        delay(40);
      }
      for(x = 9; x >= 2; x--)
      {
        digitalWrite(x, LOW);
        delay(40);
      }
    }  
  }
 
  //=================MODE 9 ==================
  if (mode==9)
  {
    runAllSequences();
  }

  //=================MODE 10 ==================
  if (mode==10)
  {
  }
}

// =========================================
// =============== FUNCTIONS ===============
// =========================================

void detail()
{
  // read the analog in value and map it to the range for the time
  if ((mode == 8) || (mode == 0)) //if not in Sound mode, or if program has not just begun
  {
    // do nothing
  }
  else
  {
    if ((potValue != analogRead(analogInPin))) //pot is not being changed
    {
      time = map(potValue, 0, 1024, 40, 200);
      // print the results to the serial monitor:
      Serial.print("\n Potentiometer = " );                      
      Serial.print(potValue);     
      Serial.print("\t Time Delay = ");     
      Serial.print(time);  
      Serial.print(" milliseconds");
      // wait 2 milliseconds before the next loop
      // for the analog-to-digital converter to settle
      // after the last reading:
      delay(2);   
      potValue = analogRead(analogInPin);
    }
  }
  // read the state of the pushbutton value:
  buttonState = digitalRead(buttonPin);
  if (buttonState == HIGH)
  {
    buttonPressed();
    digitalWrite(ledPin, LOW);
  }   
}

void detailSound()
{
  soundPotThresholdValue = analogRead(analogInPin); //threshold
  soundSensitivityLimit = map(soundPotThresholdValue, 0, 1024, 0, 512);
  soundVal=analogRead(analogInPinSound); //actual sound value
  if (soundVal > soundSensitivityLimit)
  {
    Serial.print("\n Threshold Level =   " );
    Serial.println(soundSensitivityLimit);   
    Serial.print("\n Sound Input Level = " );
    Serial.println(soundVal);    
    digitalWrite(ledPin, HIGH);     
    for(x = 2; x <= 9; x++)
    {
      digitalWrite(x, HIGH);
      delay(15);
    }
    for(x = 2; x <= 9; x++)
    {
      digitalWrite(x, LOW);
      delay(15);
    }
  }
  digitalWrite(ledPin, LOW);
  delay(10);
}

void buttonPressed()
{     
  digitalWrite(ledPin, HIGH);
  delay(300);             // pause to control the program. ensuring single mode change
  mode = mode + 1;        // increment Mode
  if (mode>9)             // if mode is greater than 9, restart from mode 1
  {
    mode=1;
    Serial.println(""); 
    Serial.println("Resetting Mode Sequence");
    Serial.println(""); 
    for (x=0;x<=3;x++)
    { 
      digitalWrite(ledPin, LOW);
      delay(100);
      digitalWrite(ledPin, HIGH);
      delay(100);
      digitalWrite(ledPin, LOW);
    }
  } 
  Serial.println("");
  Serial.println("");    
  Serial.println("Button Pressed");     
  Serial.print("Mode: ");     
  Serial.print(mode);     
  Serial.println("");
  for(x=2;x<=9;x++)        //turn all channels off - reset for next loop
  {
    digitalWrite(x, LOW);         
  }
  //if mode is sound mode, blink led 10 times quickly
  if (mode==8) //sound
  {
    for(x=0;x<=10;x++)        //turn all channels off - reset for next loop
    {
      digitalWrite(ledPin, HIGH);
      delay(50);
      digitalWrite(ledPin, LOW);
      delay(25);
    }
  }     
}
void runAllSequences()
{
  //=================MODE 1(step through all 8 channels)==================
  for (z = 0; z <=7; z++) //because this is relatively short sequence, do it a few times times
  { 
    // Step through all eight EL channels (pins 2 through 9)
    for (x=2; x<=9; x++)
    {
      detail();
      digitalWrite(x, HIGH);   // turn the EL channel on
      delay(time);              // wait for 1/10 second
      digitalWrite(x, LOW);    // turn the EL channel off
    }
  }
  for(x=2;x<=9;x++)        //turn all channels off - reset for next loop
  {
    digitalWrite(x, LOW);         
  }

  //=================MODE 2(fill up from bottom then empty from bottom)==================
  for (z = 0; z <=3; z++) //because this is relatively short sequence, do it a few times times
  {
    for (x=2; x<=9; x++)
    {
      detail();
      digitalWrite(x, HIGH);   // turn the EL channel on
      delay(time);        // wait for 1/10 second
    }
    for (x=2; x<=9; x++)
    {
      detail();
      digitalWrite(x, LOW);   // turn the EL channel on
      delay(time);        // wait for 1/10 second
    }
  }
  for(x=2;x<=9;x++)        //turn all channels off - reset for next loop
  {
    digitalWrite(x, LOW);         
  }

  //=================MODE 3(up down up down)==================
  for (z = 0; z <=3; z++) //because this is relatively short sequence, do it a few times times
  { 
    for (x=2; x<=9; x++)
    {
      detail();
      digitalWrite(x, HIGH);   // turn the EL channel on
      delay(time);        // wait for 1/10 second
      digitalWrite(x, LOW);    // turn the EL channel off
    }
    for (x=2; x<=9; x++)
    {
      detail();
      y=11-x;
      digitalWrite(y, HIGH);   // turn the EL channel on
      delay(time);        // wait for 1/10 second
      digitalWrite(y, LOW);    // turn the EL channel off
    }
  }
  for(x=2;x<=9;x++)        //turn all channels off - reset for next loop
  {
    digitalWrite(x, LOW);         
  }

  //=================MODE 4(sweep in bottom to top , sweep out top to bottom)==================
  for (z = 0; z <=3; z++) //because this is relatively short sequence, do it a few times times
  {
    for(x = 2; x <= 9; x++)
    {
      detail();
      digitalWrite(x, HIGH);
      delay(time);
    }
    for(x = 9; x >= 2; x--)
    {
      detail();
      digitalWrite(x, LOW);
      delay(time);
    }
  }
  for(x=2;x<=9;x++)        //turn all channels off - reset for next loop
  {
    digitalWrite(x, LOW);         
  }

  //=================MODE 5(stack and unstack)==================
  for (z = 0; z <=3; z++) //because this is relatively short sequence, do it a few times times
  {
    for (x = 2; x <= 9; x++)
    {
      for (y = 9; y >= x; y--)
      {
        digitalWrite(y, HIGH);
        delay(time); 
        digitalWrite(y, LOW);
        detail();   
      }
      digitalWrite(x, HIGH);
    }
    for (x = 9; x >= 2; x--)
    {
      for (y = x; y <= 9; y++)
      {
        digitalWrite(y, HIGH); 
        delay(time);              
        digitalWrite(y, LOW);      
        detail();
      }
    }
    digitalWrite(y, LOW);
    delay(time+20);
  }
  for(x=2;x<=9;x++)        //turn all channels off - reset for next loop
  {
    digitalWrite(x, LOW);         
  }

  //=================MODE 6(alternate flashing)==================
  for (z = 0; z <=10; z++) //because this is a short sequence, do it may times
  {
    detail();
    digitalWrite(2, HIGH);
    digitalWrite(4, HIGH);
    digitalWrite(6, HIGH);
    digitalWrite(8, HIGH);
    delay(time+75);
    detail();
    digitalWrite(2, LOW);
    digitalWrite(4, LOW);
    digitalWrite(6, LOW);
    digitalWrite(8, LOW);
    digitalWrite(3, HIGH);
    digitalWrite(5, HIGH);
    digitalWrite(7, HIGH);
    digitalWrite(9, HIGH);
    delay(time+75);
    detail();
    digitalWrite(3, LOW);
    digitalWrite(5, LOW);
    digitalWrite(7, LOW);
    digitalWrite(9, LOW);
  }
  for(x=2;x<=9;x++)        //turn all channels off - reset for next loop
  {
    digitalWrite(x, LOW);         
  }

  //=================MODE 7(criss cross)==================
  for (z = 0; z <=10; z++) //because this is a short sequence, do it may times
  {
    for (x = 2; x <= 9; x++)
    {
      y=11-x;
      digitalWrite(x, HIGH);   // turn the LED on (HIGH is the voltage level)

      digitalWrite(y, HIGH);   // turn the LED on (HIGH is the voltage level)
      if (x==5)
      {
        delay(time/2);
      }
      else
      {
        delay(time);
      }
      digitalWrite(x, LOW);
      digitalWrite(y, LOW);
      detail();   
    }   
  }
  for(x=2;x<=9;x++)        //turn all channels off - reset for next loop
  {
    digitalWrite(x, LOW);         
  }
}

